import React, { useEffect, useRef, useReducer } from "react";
import { Button, Tooltip, Row, Divider } from "antd";
import AutoTable from "@/components/AutoTable";
import getPrem from "@/utils/getPrem"; //权限判断fn
import { useRequest } from "umi";
import defaultFields from "./fields";
import { doFetch } from "@/utils/doFetch";
import DrawInitForm from "@/components/DrawInitForm";
import Details from "@/components/Details";
import { workingProcedureDetail } from "@/utils/detailTotalCard";
import Coltext from "@/components/Coltext";
const initState = {
    vs: false,
    fields: {},
    iftype: {},
    curitem: {},
    detail: {
      dataSource: {},
      totalCard: [],
    },
    visible: false,
  },
  otherBasic = {
    processLineName: "工艺路线",
    collectionSiteName: "采集站点",
  };
function reducer(state, action) {
  let { type } = action,
    newState = {};
  switch (type) {
    case "add":
      newState = {
        ...state,
        vs: true,
        iftype: {
          title: "新增工序",
          val: type,
        },
        fields: { ...action.fields },
      };
      break;
    case "edit":
      newState = {
        ...state,
        vs: true,
        iftype: {
          title: "编辑工序",
          val: type,
        },
        fields: { ...action.fields },
        curitem: action.curitem,
      };
      break;
    case "see":
      newState = {
        ...state,
        detail: {
          dataSource: action.dataSource,
          totalCard: [...workingProcedureDetail],
        },
        visible: true,
      };
      break;
    case "close":
      newState = {
        vs: false,
        fields: {},
        iftype: {},
        curitem: {},
        detail: {
          dataSource: {},
          totalCard: [],
        },
        visible: false,
      };
      break;
  }

  return newState;
}

const Process = (props) => {
  let actionRef = useRef(),
    ChildRef = null;
  function reload() {
    actionRef?.current?.reload();
    ChildRef?.onRefresh();
  }
  const { run, loading } = useRequest(doFetch, {
      manual: true,
      formatResult: (res) => res,
      onSuccess: (result, params) => {
        if (result.code == "0000") {
          reload();
          dispatch({ type: "close" });
        }
      },
    }),
    [state, dispatch] = useReducer(reducer, initState),
    { vs, fields, iftype, curitem, detail, visible } = state,
    columns = [
      {
        title: "工序编号",
        dataIndex: "workingProcedureCode",
        key: "workingProcedureCode",
      },
      {
        title: "工序名称",
        dataIndex: "workingProcedureName",
        key: "workingProcedureName",
        render: (_, row) => {
          return (
            <Tooltip title={row.workingProcedureName}>
              <a
                className="table-cell"
                onClick={() => {
                  doFetch({
                    url: "/ngic-workmanship/workingProcedure/queryDetailById",
                    params: { id: row.id },
                  }).then((res) => {
                    if (res.code == "0000") {
                      let dataSource = res?.data?.data ?? {};
                      dispatch({ type: "see", dataSource });
                    }
                  });
                }}
              >
                {row.workingProcedureName}
              </a>
            </Tooltip>
          );
        },
      },
      {
        title: "所属工厂",
        dataIndex: "factoryName",
        key: "factoryName",
      },
      {
        title: "所属车间",
        dataIndex: "shopName",
        key: "shopName",
      },
      {
        title: "所属工段",
        dataIndex: "sectionName",
        key: "sectionName",
      },
      {
        title: "工序类型",
        dataIndex: "workingProcedureTypeName",
        key: "workingProcedureTypeName",
        options: [
          {
            label: "生产",
            value: 1,
          },
          {
            label: "检验",
            value: 2,
          },
          {
            label: "包装",
            value: 3,
          },
        ],
        valueType: "select",
        formItemProps: {
          name: "workingProcedureType",
        },
      },
      {
        title: "采集方式",
        dataIndex: "collectionModeName",
        key: "collectionModeName",
        options: [
          {
            label: "上下线",
            value: 1,
          },
          {
            label: "一次",
            value: 2,
          },
          {
            label: "投料",
            value: 3,
          },
        ],
        valueType: "select",
        formItemProps: {
          name: "collectionMode",
        },
      },
      {
        title: "描述",
        dataIndex: "remark",
        key: "remark",
        search: false,
      },
      {
        title: "操作",
        dataIndex: "option_dataindex",
        key: "option_dataindex",
        valueType: "option",
        width: 135,
        render: (text, row, _, action) => extraAction(text, row, _, action),
      },
    ];
  function extraAction(text, record, _, action) {
    return [
      getPrem("sysDepartment_save", action, "修改", () => {
        doFetch({
          url: "/ngic-workmanship/workingProcedure/queryDetailById",
          params: { id: record.id },
        }).then((res) => {
          if (res.code == "0000") {
            let data = res?.data?.data ?? {};
            for (let i in defaultFields) {
              defaultFields[i].value = record[i];
              if (i == "factoryId" || i == "shopId") {
                if (data?.processLineName || data?.collectionSiteName) {
                  defaultFields[i].disabled = true;
                } else {
                  defaultFields[i].disabled = false;
                }
              }
            }
            dispatch({ type: "edit", fields: defaultFields, curitem: data });
          }
        });
      }),
      getPrem("sysDepartment_deleteById", action, "删除", null, {
        title: "确认删除该工序？",
        onConfirm: () => {
          run({
            url: "/ngic-workmanship/workingProcedure/deleteById",
            params: { id: record.id },
          });
        },
      }),
    ];
  }
  let saveData = (values, fn) => {
    let newfields = JSON.parse(JSON.stringify(values));
    //新增&修改
    let difrid = iftype.val == "edit" ? { id: curitem.id } : {};
    run({
      url: "/ngic-workmanship/workingProcedure/save",
      params: { ...newfields, ...difrid },
    });
  };
  let extrarender = [
    <Button
      disabled={!getPrem("sysDepartment_save", "ifs")}
      type="primary"
      onClick={() => {
        for (let i in defaultFields) {
          defaultFields[i].value = null;
          defaultFields[i].disabled = false;
        }
        dispatch({ type: "add", fields: defaultFields });
      }}
    >
      新增
    </Button>,
  ];
  return (
    <div>
      <AutoTable
        pagetitle={props.route.name} //页面标题
        pageextra={extrarender} //页面操作 新增or批量删除
        columns={columns}
        path="/ngic-workmanship/workingProcedure/queryList"
        actionRef={actionRef}
        onRef={(node) => (ChildRef = node)}
      ></AutoTable>

      <DrawInitForm
        title={iftype.title}
        visible={vs}
        onClose={() => dispatch({ type: "close" })}
        footer={false}
        destroyOnClose={true}
        fields={fields}
        submitData={(values) => {
          saveData(values);
        }}
        onChange={(changedValues, allValues) => {
          //联动操作
        }}
        submitting={loading || !vs}
        width={"50%"}
      >
        {iftype.val == "edit" && (
          <>
            <Row gutter={[16, 10]}>
              {Object.keys(otherBasic).map((it) => {
                return (
                  <Coltext
                    label={otherBasic[it]}
                    value={curitem[it]}
                    key={it}
                  />
                );
              })}
            </Row>
            <Divider />
          </>
        )}
      </DrawInitForm>
      <Details
        title="工序详情"
        visible={visible}
        onClose={() => dispatch({ type: "close" })}
        footer={false}
        destroyOnClose={true}
        width={"100%"}
        {...detail}
      ></Details>
    </div>
  );
};
export default Process;
